/* The <limits.h> header defines some basic sizes, both of the language types 
 * (e.g., the number of bits in an integer), and of the operating system (e.g.
 * the number of characters in a file name.
 */

#ifndef _LIMITS_H
#define _LIMITS_H

/* Definitions about chars (8 bits in MINIX, and signed). */
#define CHAR_BIT           8	/* # bits in a char */
#define CHAR_MIN        -128	/* minimum value of a char */
#define CHAR_MAX         127	/* maximum value of a char */
#define SCHAR_MIN       -128	/* minimum value of a signed char */
#define SCHAR_MAX        127	/* maximum value of a signed char */
#define UCHAR_MAX        255	/* maximum value of an unsigned char */
#define MB_LEN_MAX         1	/* maximum length of a multibyte char */

/* Definitions about shorts (16 bits in MINIX). */
#define SHRT_MIN  (-32767-1)	/* minimum value of a short */
#define SHRT_MAX       32767	/* maximum value of a short */
#define USHRT_MAX     0xFFFF	/* maximum value of unsigned short */

#define INT_MIN (-2147483647-1)	/* minimum value of a 32-bit int */
#define INT_MAX   2147483647	/* maximum value of a 32-bit int */
#define UINT_MAX  0xFFFFFFFF	/* maximum value of an unsigned 32-bit int */
#define UINT_MIN (-2147483647L-1)/* minimum value of a long */

#include "sys/dir.h"


/* Values actually implemented by MINIX (Tables 2-4, 2-5, 2-6, and 2-7). */
/* Some of these old names had better be defined when not POSIX. */
#define _NO_LIMIT          100	/* arbitrary number; limit not enforced */
#define CHILD_MAX    _NO_LIMIT	/* MINIX does not limit children */
#define OPEN_MAX            20	/* # open files a process may have */
//#if 0			/* V1 file system */
//#define LINK_MAX      CHAR_MAX	/* # links a file may have */
//#else			/* V2 or better file system */
//#define LINK_MAX      SHRT_MAX	/* # links a file may have */
//#endif
#define NAME_MAX        DIRSIZ	/* # chars in a file name */
#define PATH_MAX           255	/* # chars in a path name */
#define PIPE_BUF          7168	/* # bytes in atomic write to a pipe */
#define STREAM_MAX          20	/* must be the same as FOPEN_MAX in stdio.h */
#define TZNAME_MAX           3	/* maximum bytes in a time zone name is 3 */
#define SSIZE_MAX        32767	/* max defined byte count for read() */

//#endif /* _POSIX_SOURCE */

#endif /* _LIMITS_H */
