#include "libce.h"
#include "fs.h"
#include "super.h"
#include "blocco_fis.h"

extern "C" void c_readhd_n(uint16_t vetti[], uint32_t primo,
		uint8_t quanti, uint8_t &errore);
extern "C" void c_dmareadhd_n(uint16_t vetti[], uint32_t primo,
		uint16_t quanti, uint8_t &errore);
extern "C" void c_dmawritehd_n(uint16_t vetti[], uint32_t primo,
		uint16_t quanti, uint8_t &errore);

int read_bytes_hd(dev_t dev, block_t pos, void *buf, uint32_t nbytes)
{
	super_block *sp;
	uint16_t block_size, n_blocks;
	void *my_buf;
	uint8_t errore;

	if( (sp = get_super(dev)) == NIL_SUPER )
		return -1;
	block_size = sp->s_block_size;

	n_blocks = nbytes/block_size + (nbytes%block_size != 0);
	my_buf = mem_alloc(n_blocks*block_size);
	
	// lo heap è residente
	c_dmareadhd_n((uint16_t*)my_buf, SECTOR_OFF(pos), NUM_SECTORS(n_blocks), errore);
	memcpy(buf,my_buf,nbytes);
	mem_free(my_buf);
	//manca la gestione dell'errore !
	return nbytes;
}

int write_bytes_hd(dev_t dev, block_t pos, void *buf, uint32_t nbytes)
{
	super_block *sp;
	uint16_t block_size, n_blocks;
	void *my_buf;
	uint8_t errore;

	if( (sp = get_super(dev)) == NIL_SUPER )
		return -1;
	block_size = sp->s_block_size;

	n_blocks = nbytes/block_size + (nbytes%block_size != 0);
	// lo heap è residente
	my_buf = mem_alloc(n_blocks*block_size);
	memcpy(my_buf,buf,nbytes);
	
	c_dmawritehd_n((uint16_t*)my_buf, SECTOR_OFF(pos), NUM_SECTORS(n_blocks), errore);
	mem_free(my_buf);
 	//manca la gestione dell'errore !
	return nbytes; 
}

blocco_fis *get_block(dev_t dev, block_t block, int only_search)
{
	blocco_fis *bp = (blocco_fis*)mem_alloc(sizeof(blocco_fis));
	uint8_t errore;

	if( dev!=NO_DEV && block!=NO_BLOCK && only_search==NORMAL)
	{
		c_dmareadhd_n((natw*)bp,SECTOR_OFF(block),NUM_SECTORS(1),errore);
	}

	return bp;
}

void put_block(dev_t dev, block_t block, blocco_fis *bp)
{
	uint8_t errore;

	if( dev!=NO_DEV && block!=NO_BLOCK )
	{
		c_dmawritehd_n((natw*)bp,SECTOR_OFF(block),NUM_SECTORS(1),errore);
	}

	//manca gestione dell'errore
}

blocco_fis *bloccofis_file(inode *rip, off_t position)
{
	blocco_fis *bp;
	/*unsigned index_zone = position/ZONE_SIZE;
	if( index_zone >= V2_NR_DZONES)
		return NIL_BLK;

	// individuo la zona
	zone_t z = rip->i_zone[index_zone];
	// individuo il blocco fisico
	block_t index_block = z + (position%ZONE_SIZE) / STATIC_BLOCK_SIZE*/
	bp = get_block(rip->i_dev,read_map(rip,position),NORMAL);
	flog(LOG_DEBUG,"bloccofis lettura da %d position:%d",read_map(rip,position),position);
	return bp;
}
