#ifndef _FS_BLOCCO_FIS
#define _FS_BLOCCO_FIS

#include "const.h"
#include "sys/dir.h"
// blocco fisico
// è ricavato dalla definizione presente nei
// sorgenti originali di minix in fs/buf.h
// il filesystem da me realizzato non ha cache dei blocchi(per adesso)


union blocco_fis {
    char b_data[BLOCK_SIZE];		     /* ordinary user data */
// directory block con nomi di 14B
    direct_V1 b_dirV1[NR_DIR_ENTRIES(BLOCK_SIZE,16)];    
// directory block con nomi di 30B 
    direct_V1_REV b_dirV1_REV[NR_DIR_ENTRIES(BLOCK_SIZE,32)]; 
// directory block con nomi di 60B
    direct_V3 b_dir[NR_DIR_ENTRIES(BLOCK_SIZE,64)];
/* V1 indirect block */
    zone1_t b_v1_ind[V1_INDIRECTS];	     
/* V2 indirect block */
    zone_t  b_v2_ind[V2_INDIRECTS(BLOCK_SIZE)];	     
/* V1 inode block */
    d1_inode b_v1_ino[V1_INODES_PER_BLOCK]; 
/* V2 inode block */
    d2_inode b_v2_ino[V2_INODES_PER_BLOCK(BLOCK_SIZE)]; 
/* bit map block */
    bitchunk_t b_bitmap[FS_BITMAP_CHUNKS(BLOCK_SIZE)];  
 };
/*
 */

// Astraggono la lettura/scrittura dall'hard disk.
// pos: posizione del blocco da cui leggere
// buf: buffer fornito dal processo
// nbytes: numero di byte da leggere
// N.B. La dimensione del blocco in minix è 1KiB
int read_bytes_hd(dev_t dev, block_t pos, void *buf, uint32_t nbytes);
int write_bytes_hd(dev_t dev, block_t pos, void *buf, uint32_t nbytes);
blocco_fis *get_block(dev_t dev, block_t block, int only_search);
void put_block(dev_t dev, block_t block, blocco_fis *bp);
blocco_fis *bloccofis_file(inode *rip, off_t position);

//( originariamente NIL_BUF 
#define NIL_BLK ((blocco_fis *) 0)      /* indicates absence of a buffer */
// )
#define INODE_BLOCK        0                             /* inode block */
#define DIRECTORY_BLOCK    1                             /* directory block */
#define INDIRECT_BLOCK     2                             /* pointer block */
#define MAP_BLOCK          3                             /* bit map */
#define FULL_DATA_BLOCK    5                             /* data, fully used */
#define PARTIAL_DATA_BLOCK 6                             /* data, partly used*/
#endif
