#ifndef FS_CONST_H
#define FS_CONST_H

#define MAX_FILE_SIZE NR_DZONES*ZONE_SIZE + ((DIM_BLOCK/4)*ZONE_SIZE) + ((DIM_BLOCK/4)*(DIM_BLOCK/4)*ZONE_SIZE)
#define INODE_SIZE sizeof(d2_inode)

#define CEILING_POS(X) ((X-(int)(X)) > 0 ? (int)(X+1) : (int)(X))
#define CEILING_NEG(X) ((X-(int)(X)) < 0 ? (int)(X-1) : (int)(X))
#define CEILING(X) ( ((X) > 0) ? CEILING_POS(X) : CEILING_NEG(X) )

/* Tables sizes */

#define NR_DZONES       7	/* # direct zone numbers in a V2 inode */
#define NR_TZONES      10	/* total # zone numbers in a V2 inode */

#define V1_NR_DZONES       7    /* # direct zone numbers in a V1 inode */
#define V1_NR_TZONES       9    /* total # zone numbers in a V1 inode */

#define V2_NR_DZONES   7
#define V2_NR_TZONES   10

#define NR_FILPS         128	/* # slots in filp table */
#define NR_INODES         64	/* # slots in "in core" inode table */
#define NR_SUPERS          1	/* # slots in super block table */
#define NR_LOCKS           8	/* # slots in the file locking table */
#define ZONE_SIZE	    1024 

/* The type of sizeof may be (unsigned) long.  Use the following macro for
 * taking the sizes of small objects so that there are no surprises like
 * (small) long constants being passed to routines expecting an int.
 */
#define usizeof(t) ((unsigned) sizeof(t))

/* File system types. */
#define MINIX_SUPER_MAGIC	0x137F		/* minix v1 fs, 14 char names */
#define MINIX_SUPER_MAGIC2	0x138F		/* minix v1 fs, 30 char names */
#define MINIX2_SUPER_MAGIC	0x2468		/* minix v2 fs, 14 char names */
#define MINIX2_SUPER_MAGIC2	0x2478		/* minix v2 fs, 30 char names */
#define MINIX3_SUPER_MAGIC	0x4d5a		/* minix v3 fs, 60 char names */

#ifndef VERS_FS
#define VERS_FS
#define V1		  0x11	// filesystem V1, nomi file di 14B
#define V1_REV	  0x12  // filesystem V1, nomi file di 30B
#define V2		  0x21	// filesystem V2, nomi file di 14B
#define V2_REV	  0x22 	// filesystem V2, nomi file di 30B
#define V3		  0x32	// filesystem V3, nomi di 60B
#endif

/* Miscellaneous constants */
#define SU_UID 	 ((uid_t) 0)	/* super_user's uid_t */
#define SYS_UID  ((uid_t) 0)	/* uid_t for processes MM and INIT */
#define SYS_GID  ((gid_t) 0)	/* gid_t for processes MM and INIT */
#define NORMAL	           0	/* forces get_block to do disk read */
#define NO_READ            1	/* prevents get_block from doing disk read */
#define PREFETCH           2	/* tells get_block not to read or mark dev */

#define NO_BIT   ((bit_t) 0)	/* returned by alloc_bit() to signal failure */
#define DUP_MASK        0100	/* mask to distinguish dup2 from dup */

#define LOOK_UP            0 /* tells search_dir to lookup string */
#define ENTER              1 /* tells search_dir to make dir entry */
#define DELETE             2 /* tells search_dir to delete entry */
#define IS_EMPTY           3 /* tells search_dir to ret. OK or ENOTEMPTY */  

#define CLEAN              0	/* disk and memory copies identical */
#define DIRTY              1	/* disk and memory copies differ */
#define ATIME            002	/* set if atime field needs updating */
#define CTIME            004	/* set if ctime field needs updating */
#define MTIME            010	/* set if mtime field needs updating */

#define BYTE_SWAP          0	/* tells conv2/conv4 to swap bytes */

#define END_OF_FILE   (-104)	/* eof detected */

#define ROOT_INODE         1		/* inode number for root directory */
#define BOOT_BLOCK  ((block_t) 0)	/* block number of boot block */
#define BOOT_BLOCK_BYTES  (1024)
#define SUPER_BLOCK_BYTES (1024)	/* bytes offset */
#define START_BLOCK 	2		/* first block of FS (not counting SB) */


// ( Attenzione la macro è valida solo per minix v3 !
//#define DIR_ENTRY_SIZE       usizeof (struct direct)  /* # bytes/dir entry   */
#define NR_DIR_ENTRIES(b, entry_size)   ((b)/(entry_size))  /* # dir entries/blk   */
// )
#define SUPER_SIZE      usizeof (struct super_block)  /* super_block size    */
#define PIPE_SIZE(b)          (V1_NR_DZONES*(b))  /* pipe size in bytes  */

#define FS_BITMAP_CHUNKS(b) ((b)/usizeof (bitchunk_t))/* # map chunks/blk   */
#define FS_BITCHUNK_BITS		(usizeof(bitchunk_t) * CHAR_BIT)
#define FS_BITS_PER_BLOCK(b)	(FS_BITMAP_CHUNKS(b) * FS_BITCHUNK_BITS)

/* Derived sizes pertaining to the V1 file system. */
#define V1_ZONE_NUM_SIZE           usizeof (zone1_t)  /* # bytes in V1 zone  */
#define V1_INODE_SIZE             usizeof (d1_inode)  /* bytes in V1 dsk ino */

/* # zones/indir block */
#define V1_INDIRECTS (BLOCK_SIZE/V1_ZONE_NUM_SIZE)  

/* # V1 dsk inodes/blk */
#define V1_INODES_PER_BLOCK (BLOCK_SIZE/V1_INODE_SIZE)

/* Derived sizes pertaining to the V2 file system. */
#define V2_ZONE_NUM_SIZE            usizeof (zone_t)  /* # bytes in V2 zone  */
#define V2_INODE_SIZE             usizeof (d2_inode)  /* bytes in V2 dsk ino */
#define V2_INDIRECTS(b)   ((b)/V2_ZONE_NUM_SIZE)  /* # zones/indir block */
#define V2_INODES_PER_BLOCK(b) ((b)/V2_INODE_SIZE)/* # V2 dsk inodes/blk */

//Minix gestisce il disco come un'array di blocchi la cui
//dimensione minima è di 1KiB mentre la dimensione del blocco
//fisico(settore) è di 512B.
#define BLOCK_SIZE 1024 //dimensione blocco per minix
#ifndef DIM_SECTOR
#define DIM_SECTOR 512
#endif
#define NUM_SECTORS(X) ((X)*(BLOCK_SIZE/DIM_SECTOR)) // converte in settori fisici
#define SECTOR_OFF(X) ((X)*(BLOCK_SIZE/DIM_SECTOR)) //offset in blocchi fisici

#define BOOT_BLOCK_OFFSET 0
#define SUPER_BLOCK_OFFSET 1

// nel nostro nucleo abbiamo un solo disco
#define CE_DEV      1


#endif
