/* This file contains the procedures that manipulate file descriptors.
 *
 * The entry points into this file are
 *   get_fd:	look for free file descriptor and free filp slots
 *   get_filp:	look up the filp entry for a given file descriptor
 *   find_filp:	find a filp slot that points to a given inode
 */
#include "libce.h"
#include "fs.h"
#include "file.h"
#include "fproc.h"
#include "inode.h"
#include "glo.h"

/*===========================================================================*
 *				get_fd					     *
 *===========================================================================*/
PUBLIC int get_fd(int start, mode_t bits, int *k, filp **fpt)
{
/* Look for a free file descriptor and a free filp slot.  Fill in the mode word
 * in the latter, but don't claim either one yet, since the open() or creat()
 * may yet fail.
 */

	sem_wait(fs_glo_mutex); // modifico delle strutture dati condivise
	fproc *fp = get_fproc(getpid()); //il nostro kernel è monilitico
	filp *f;
	int i;

	*k = -1;			// we need a way to tell if file desc found 

	// Search the fproc fp_filp table for a free file descriptor. */
	for (i = start; i < OPEN_MAX; i++) {
		if (fp->fp_filp[i] == NIL_FILP) {
			// A file descriptor has been located. 
			*k = i;
			break;
		}
	}

	// Check to see if a file descriptor has been found. 
	if (*k < 0){
		sem_signal(fs_glo_mutex);
		return(EMFILE);	// this is why we initialized k to -1 
	}

	// Now that a file descriptor has been found, look for a free filp slot. 
	for (f = &filps_table[0]; f < &filps_table[NR_FILPS]; f++) {
		if (f->filp_count == 0) {
			f->filp_mode = bits;
			f->filp_pos = 0L;
			//f->filp_selectors = 0;
			//f->filp_select_ops = 0;
			//f->filp_pipe_select_ops = 0;
			f->filp_flags = 0;
			*fpt = f;
			sem_signal(fs_glo_mutex);
			return(OK);
		}
	}

	sem_signal(fs_glo_mutex);
	// If control passes here, the filp table must be full.  Report that back. 
	return(ENFILE);
}

/*===========================================================================*
 *				get_filp				     *
 *===========================================================================*/
PUBLIC filp *get_filp(int fild)
//int fild;			/* file descriptor */
{
// See if 'fild' refers to a valid file descr.  If so, return its filp ptr. 
	fproc *fp = get_fproc(getpid());
	err_code = EBADF;
	//flog(LOG_DEBUG,"get_filp fp:%d fild:%d",fp,fild);
	if (fild < 0 || fild >= OPEN_MAX ) return(NIL_FILP);
	//flog(LOG_DEBUG,"get_filp fild:%d fp->fp_filp[fild]:%d",fild,fp->fp_filp[fild]);
	return(fp->fp_filp[fild]);	/* may also be NIL_FILP */
}

/*===========================================================================*
 *				find_filp				     *
 *===========================================================================*/
PUBLIC filp *find_filp(inode *rip, mode_t bits)
{
/* Find a filp slot that refers to the inode 'rip' in a way as described
 * by the mode bit 'bits'. Used for determining whether somebody is still
 * interested in either end of a pipe.  Also used when opening a FIFO to
 * find partners to share a filp field with (to shared the file position).
 * Like 'get_fd' it performs its job by linear search through the filp table.
 */
	filp *f;

	sem_signal(fs_glo_mutex);
	for (f = &filps_table[0]; f < &filps_table[NR_FILPS]; f++) {
		if (f->filp_count != 0 && f->filp_ino == rip && (f->filp_mode & bits)){
			sem_signal(fs_glo_mutex);
			return(f);
		}
	}

	sem_signal(fs_glo_mutex);
	// If control passes here, the filp wasn't there.  Report that back. 
	return(NIL_FILP);
}
