#include "libce.h"
#include "fs.h"
#include "glo.h"

extern "C" void c_dmareadhd_n(uint16_t vetti[], uint32_t primo,
		uint16_t quanti, uint8_t &errore);

PUBLIC
void read_super_blockCE()
{
	// in questo sistema abbiamo un solo disco
	super_b->s_dev = CE_DEV;
	flog(LOG_DEBUG,"read_super_blockCE");
	int ris = read_super(super_b);
	if( ris != OK )
		flog(LOG_ERR,"errore lettura superblocco");
	/*flog(LOG_DEBUG,"super_block letto \ns_block_size:%d \ns_log_zone_size:%d \ns_imap_blocks:%d \ns_zmap_blocks:%d \ns_dev:%d \ns_firstdatazone %d \ns_version:%d",
		super_b->s_block_size,super_b->s_log_zone_size,super_b->s_imap_blocks,super_b->s_zmap_blocks,super_b->s_dev,super_b->s_firstdatazone,super_b->s_version);*/
}

PRIVATE
void read_bit_maps()
{
	natb errore;
	inode_map = (char*)mem_alloc(BLOCK_SIZE*super_b->s_imap_blocks);
	zone_map = (char*)mem_alloc(BLOCK_SIZE*super_b->s_zmap_blocks);

	//leggo la inode bitmap
	//c_dmareadhd_n ((natw*)inode_map, __imap_offset(), super_b->s_imap_blocks, errore);
	c_dmareadhd_n((natw*)inode_map, __imap_offset(), NUM_SECTORS(super_b->s_imap_blocks),errore);
	//leggo la zone bitmap
	c_dmareadhd_n((natw*)zone_map, __zmap_offset(), NUM_SECTORS(super_b->s_zmap_blocks),errore);
	flog(LOG_DEBUG,"bitmaps lette");
}

PRIVATE
void read_inode_table()
{
/*	natb errore;
	if( inodes_table == NULL )
		inodes_table = (inode*)mem_alloc(__n_blocks_inode_table()*DIM_BLOCK);
	//flog(LOG_DEBUG,"Il n. di blocchi della inodes_table è %d",__n_blocks_inode_table());
	c_dmareadhd_n((natw*)inodes_table,__inode_table_offset(__data_size()),__n_blocks_inode_table(),errore);
	flog(LOG_DEBUG,"letta tab inode. indirizzo=%x",inodes_table);*/
	memset(inodes_table,0,sizeof(inode)*NR_INODES);
	//flog(LOG_DEBUG,"indirizzo &inodes_table=%x inodes_table=%x",&inodes_table,inodes_table);
	//leggo solo la root
	flog(LOG_DEBUG,"leggo il root inode");
	get_inode(CE_DEV,ROOT_INODE);
}

PRIVATE
void fs_fork(pid_t pid, pid_t ppid)
{
	fproc *fp_padre = get_fproc(ppid);
	fproc *fp_figlio = get_fproc(pid);
	memcpy(fp_figlio,fp_padre,sizeof(fproc));
	//flog(LOG_DEBUG, "c_fs_fork");
	for(int i=0; i<OPEN_MAX; i++)
	{
		// scorro la tabella dei descrittori dei file aperti
		// se l'entrata è un file aperto allora incremento il
		// contatore filp_count ed il contatore dell'inode puntato
		filp *f_figlio = fp_figlio->fp_filp[i];
		if( f_figlio ){
			// c'è un processo in più che usa quell'inode
			f_figlio->filp_ino->i_count++;
			f_figlio->filp_count++;
		}
	}
}

PUBLIC
extern "C" void c_fs_init()
{
	pid_t pid = getpid();
	pid_t ppid = getppid();
	flog(LOG_DEBUG,"!!!!c_fs_init pid:%d ppid:%d",pid,ppid);
	sem_wait(fs_glo_mutex);
	if( !n_fs_init )
	{
		read_super_blockCE();
		flog(LOG_DEBUG,"leggo bit_maps");
		read_bit_maps();
		flog(LOG_DEBUG,"leggo inode table");
		read_inode_table();
		init_fproc_table();
		init_filps_table();
	}
	n_fs_init++;
	
	//flog(LOG_DEBUG,"n_fs_init:%d",n_fs_init);

	// il processo non è stato creato tramite fork
	if( ppid == 0 ) 
	{
		fproc *fp = get_fproc(pid);
		
		fp->fp_realuid = getuid();
		fp->fp_realgid = getgid();
		fp->fp_rootdir = get_inode(CE_DEV,ROOT_INODE);
		fp->fp_workdir = get_inode(CE_DEV,ROOT_INODE);
		fp->fp_umask = ALL_MODES;
		for(filp *f=fp->fp_filp[0]; f<fp->fp_filp[OPEN_MAX]; f++)
			f = NIL_FILP;
	} else {
		fs_fork(pid,ppid);
	}
	sem_signal(fs_glo_mutex);
}

PUBLIC
extern "C" void c_fs_flush()
{
	/*for(inode *i=&inodes_table[0]; i<&inodes_table[NR_INODES]; i++)
	{
		//flog(LOG_DEBUG,"c_fs_flush i_dev:%d i_num:%d i_nlinks:%d",i->i_dev,i->i_num,i->i_nlinks);
		if( i->i_dev != NO_DEV ) 
			rw_inode(i,WRITING);
	}*/
	//flog(LOG_DEBUG,"c_fs_flush n_fs_init:%d",n_fs_init);
	// tabella dei file aperti del processo
//	sem_wait(fs_glo_mutex);
	if( !n_fs_init )
		return;

	filp **filps_p = get_fproc(getpid())->fp_filp;
	filp *fp;
	for(int i=0; i<OPEN_MAX; i++) {
		fp = filps_p[i];
	//	flog(LOG_DEBUG,"c_fs_flush fp:%x",fp);
		if(fp != NIL_FILP ){
			put_inode(fp->filp_ino);
		}
	}
	n_fs_init--;

	//l'ultimo scrive l'inode della directory root
	if( !n_fs_init )
		rw_inode(get_inode(CE_DEV,ROOT_INODE),WRITING);
//	sem_signal(fs_glo_mutex);
}
