/* This file deals with protection in the file system.  It contains the code
 * for four system calls that relate to protection.
 *
 * The entry points into this file are
 *   do_chmod:	perform the CHMOD system call
 *   do_chown:	perform the CHOWN system call
 *   do_umask:	perform the UMASK system call
 *   do_access:	perform the ACCESS system call
 *   forbidden:	check to see if a given access is allowed on a given inode
 */
#include "libce.h"
#include "fs.h"
//#include <unistd.h>
//#include <minix/callnr.h>
//#include "buf.h"
#include "file.h"
#include "fproc.h"
#include "inode.h"
//#include "param.h"
#include "super.h"
#include "glo.h"

/*===========================================================================*
 *				forbidden				     *
 *===========================================================================*/
PUBLIC int forbidden(inode *rip, mode_t access_desired)
{
/* Given a pointer to an inode, 'rip', and the access desired, determine
 * if the access is allowed, and if not why not.  The routine looks up the
 * caller's uid in the 'fproc' table.  If access is allowed, OK is returned
 * if it is forbidden, EACCES is returned.
 */

	inode *old_rip = rip;
	mode_t bits, perm_bits;
	int r, shift, test_uid, test_gid;
	fproc *fp = get_fproc(getpid());

  	/* Isolate the relevant rwx bits from the mode. */
	bits = rip->i_mode;
	// NB: non faccio distinzione tra realuid ed effuid
	/*test_uid = (call_nr == ACCESS ? fp->fp_realuid : fp->fp_effuid);
	test_gid = (call_nr == ACCESS ? fp->fp_realgid : fp->fp_effgid);*/
	test_uid = fp->fp_realuid;
  	test_gid = fp->fp_realgid;

  	flog(LOG_DEBUG,"test_uid:%d test_gid:%d bits:%x",test_uid,test_gid,bits);
	if (test_uid == SU_UID) {
		/* Grant read and write permission.  Grant search permission for
	 	* directories.  Grant execute permission (for non-directories) if
	 	* and only if one of the 'X' bits is set. */
		if ( (bits & I_TYPE) == I_DIRECTORY ||
			bits & ((X_BIT << 6) | (X_BIT << 3) | X_BIT))
			perm_bits = R_BIT | W_BIT | X_BIT;
		else
			perm_bits = R_BIT | W_BIT;
	} else {
		if (test_uid == rip->i_uid) shift = 6;		/* owner */
		else if (test_gid == rip->i_gid ) shift = 3;	/* group */
		else shift = 0;					/* other */
		perm_bits = (bits >> shift) & (R_BIT | W_BIT | X_BIT);
	}

	// If access desired is not a subset of what is allowed, it is refused. 
  	r = OK;
	if ((perm_bits | access_desired) != perm_bits) r = EACCES;

  	/* Check to see if someone is trying to write on a file system that is
   	* mounted read-only.
   	*/
  	if (r == OK)
		if (access_desired & W_BIT)
			r = read_only(rip);

	if (rip != old_rip) put_inode(rip);

	return(r);
}

/*===========================================================================*
 *				read_only				     *
 *===========================================================================*/
PUBLIC int read_only(inode *ip)
//struct inode *ip;		/* ptr to inode whose file sys is to be cked */
{
/* Check to see if the file system on which the inode 'ip' resides is mounted
 * read only.  If so, return EROFS, else return OK.
 */

	register super_block *sp;

  	sp = ip->i_sp;
  	return(sp->s_rd_only ? EROFS : OK);
}
