#include <lib.h>
#include <sys.h>

int printf(const char *fmt, ...)
{
	va_list ap;
	char buf[1024];
	int l;

	va_start(ap, fmt);
	l = vsnprintf(buf, 1024, fmt, ap);
	va_end(ap);

	writeconsole(buf);

	return l;
}

int sprintf(char *buf, natq size, const char *fmt, ...)
{
	va_list ap;
	int l;

	va_start(ap, fmt);
	l = vsnprintf(buf, size, fmt, ap);
	va_end(ap);

	return l;
}

char pause_buf[1];
natl pause_len = 1;
void pause()
{
#ifndef AUTOCORR
	writeconsole("Premere INVIO per continuare");
	readconsole(pause_buf, pause_len);
#endif
}


extern "C" void panic(const char* msg)
{
	flog(LOG_WARN, "%s", msg);
	terminate_p();
}


natl mem_mutex;

void* mem_alloc(natl dim)
{
	void *p;

	sem_wait(mem_mutex);
	p = alloca(dim);
	sem_signal(mem_mutex);

	return p;
}


void mem_free(void* p)
{
	sem_wait(mem_mutex);
	dealloca(p);
	sem_signal(mem_mutex);
}

extern "C" void do_log(log_sev sev, const char* msg, natl size)
{
	log(sev, msg, size);
}

extern "C" natl end;

extern "C" void lib_init()
{
	mem_mutex = sem_ini(1);
	heap_init(&end, DIM_USR_HEAP);
}

extern "C" void sys_call_opendir(const char *pathname, DIR* p);
DIR *opendir(const char* pathname)
{
	DIR *p = (DIR*)mem_alloc(sizeof(DIR));
	if( !p ) 
		return NULL;
	sys_call_opendir(pathname, p);
	return p;
}

int readdir_r(DIR *dir, direct *entry, direct **result)
{
	direct *d;
	off_t max_entry;
	off_t dim_entry;

	if( !dir || dir->fd<0 )
		return EBADF;

	if( dir->version == V1 || dir->version == V2 )
		dim_entry = 16;
	else if( dir->version == V1_REV || dir->version == V2_REV )
		dim_entry = 32;
	else
		dim_entry = 64;
	max_entry = dir->dir_size / dim_entry;
	

	if( (dir->pos/sizeof(direct)) == max_entry ){
		*result = NULL;
		return 0;
	}

	d = (direct*)mem_alloc(sizeof(direct));
	if( !d )
		return -1;
	// trovo un entrata con un inode non nullo
	while( dir->pos < dir->dir_size ){
		read(dir->fd, d, dim_entry);
		dir->pos += dim_entry;
		if( dir->version == V1 || dir->version == V2 ){
			if( ((direct_V1*)d)->d_ino != 0 ) {
				entry->d_ino = ((direct_V1*)d)->d_ino; //viene esteso su 4B
				memcpy(entry->d_name,((direct_V1*)d)->d_name,14);
				break;
			}
		} else if ( dir->version == V1_REV || dir->version == V2_REV ) {
			if( ((direct_V1_REV*)d)->d_ino != 0 ) {
				entry->d_ino = ((direct_V1_REV*)d)->d_ino; //viene esteso su 4B
				memcpy(entry->d_name,((direct_V1_REV*)d)->d_name,30);
				break;
			}
		} else { // V3
			if( d->d_ino != 0 ) {
				memcpy(entry, d, sizeof(direct));
				break;
			}
		}	
	}
	
	if( dir->pos == dir->dir_size )
		*result = NULL; 

	mem_free(d);
	return 0;
}

int closedir(DIR *dir)
{
	if( !dir )
		return -1;
	close(dir->fd);
	mem_free(dir);
	return 0;
}

void rewinddir(DIR *dir)
{
	if( !dir ) return;
	dir->pos = 0;
}
